#version 310 es
#extension GL_ANDROID_extension_pack_es31a : enable

precision mediump float;

// INPUT
in vec2 v_vTexcoord;
in vec4 v_vPosWorldSpace;
in vec4 v_vPosEyeSpace;

// OUTPUT
out vec4 o_vFragColor;

// UNIFORMS
layout(binding = 2) uniform sampler2D u_tCaustic1Tex;
layout(binding = 3) uniform sampler2D u_tCaustic2Tex;
layout(binding = 4) uniform sampler2D u_tSandTex;

layout(binding = 1) uniform LightingBlock {
	vec4 u_vLightPosition;  
	vec4 u_vLightAmbient;   
	vec4 u_vLightDiffuse;   
	float u_fCausticOffset;
	float u_fCausticTiling;
};

float getCausticIntensity(vec4 worldPos)
{
	vec2 sharedUV = (worldPos.xz * u_fCausticTiling);
	float caustic1 = texture(u_tCaustic1Tex, sharedUV + vec2(u_fCausticOffset, 0.0f)).r;
	float caustic2 = texture(u_tCaustic2Tex, sharedUV * 1.3f + vec2(0.0f, u_fCausticOffset)).r;
	return (caustic1 * caustic2);
}

float g_maxFogDist = 75.0f;
vec3 g_fogColor = vec3(0.0f, 0.0f, 0.64f);

float fogIntensity(float dist)
{
	float normalizedDist = clamp((dist / g_maxFogDist), 0.0f, 1.0f);
	// Use a linear falloff to give the ground plane more of a distant look
	return normalizedDist;// * normalizedDist * normalizedDist;
}

void main()
{
    o_vFragColor = texture(u_tSandTex, v_vTexcoord) + getCausticIntensity(v_vPosWorldSpace);
    float fog = fogIntensity(length(v_vPosEyeSpace.xyz));
    o_vFragColor.rgb = mix(o_vFragColor.rgb, g_fogColor, fog);
}
