#version 310 es
#extension GL_ANDROID_extension_pack_es31a : enable

precision mediump float;

// INPUT
in vec2 v_vTexcoord;
in vec3 v_vLightIntensity;
in vec4 v_vPosEyeSpace;
in vec4 v_vPosWorldSpace;
in vec3 v_vNormalWorldSpace;

// OUTPUT
out vec4 o_vFragColor;

// UNIFORMS
layout(binding = 2) uniform sampler2D u_tCaustic1Tex;
layout(binding = 3) uniform sampler2D u_tCaustic2Tex;
layout(binding = 0) uniform sampler2D u_tDiffuseTex;

layout(binding = 1) uniform LightingBlock {
	vec4 u_vLightPosition;  
	vec4 u_vLightAmbient;   
	vec4 u_vLightDiffuse;   
	float u_fCausticOffset;
	float u_fCausticTiling;
};

float getCausticIntensity(vec4 worldPos)
{
	vec2 sharedUV = worldPos.xz * u_fCausticTiling;
	float caustic1 = texture(u_tCaustic1Tex, sharedUV + vec2(u_fCausticOffset, 0.0f)).r;
	float caustic2 = texture(u_tCaustic2Tex, sharedUV * 1.3f + vec2(0.0f, u_fCausticOffset)).r;
	return (caustic1 * caustic2) * 2.0f;
}

float g_maxFogDist = 75.0f;
vec3 g_fogColor = vec3(0.0f, 0.0f, 0.64f);

float fogIntensity(float dist)
{
	// Push the start of the fog range out, so that we have a nice, bright range of fish
	// near the camera with a steeper falloff further out.
	dist -= 10.0f;

	float normalizedDist = clamp((dist / g_maxFogDist), 0.0f, 1.0f);

	// Also use a square distance to maintain some color from the fish vs. the ground
	return normalizedDist * normalizedDist; 
}

void main()
{
    o_vFragColor = texture(u_tDiffuseTex, v_vTexcoord);
    if (o_vFragColor.a < 0.5f)
    {
        discard;
    }
    o_vFragColor.rgb *= v_vLightIntensity + vec3(getCausticIntensity(v_vPosWorldSpace));
    float fog = fogIntensity(length(v_vPosEyeSpace.xyz));
    o_vFragColor.rgb = mix(o_vFragColor.rgb, g_fogColor, fog);
}
