// TAGRELEASE: PUBLIC
#GLSL_VS
#version 440 core

precision mediump float;

// INPUT
layout(location = 0) in vec2 a_vPosition; // In BaseShader
layout(location = 1) in vec2 a_vTexCoord; // In SceneColorShader

// OUTPUT
layout(location = 0) out vec2 v_vTexcoord;
layout(location = 1) out vec4 v_vPosWorldSpace;
layout(location = 2) out vec4 v_vPosEyeSpace;

// UNIFORMS
layout(set = 0, binding=0) uniform ProjBlock {
	mat4 u_mProjectionMatrix;
	mat4 u_mInverseProjMatrix;
	mat4 u_mViewMatrix;
	mat4 u_mInverseViewMatrix;
};

layout(set = 0, binding=1) uniform LightingBlock {
	vec4 u_vLightPosition;  
	vec4 u_vLightAmbient;   
	vec4 u_vLightDiffuse;   
	float u_fCausticOffset;
	float u_fCausticTiling;
};

void main()
{
    float size = 100.0f;
    float y = 0.0f;
	float tiling = 0.1f;

    v_vPosWorldSpace = vec4(a_vPosition.x * size, 0.0, a_vPosition.y * size, 1.0f) + u_mInverseViewMatrix[3];
	v_vPosWorldSpace.y = y;
    v_vPosWorldSpace.w = 1.0f;

	v_vTexcoord = (v_vPosWorldSpace.zx + a_vTexCoord)  * tiling;

    v_vPosEyeSpace = u_mViewMatrix * vec4(v_vPosWorldSpace.xyz, 1.0);
    gl_Position = u_mProjectionMatrix * v_vPosEyeSpace;
}

#GLSL_FS
#version 440 core

precision mediump float;

// INPUT
layout(location = 0) in vec2 v_vTexcoord;
layout(location = 1) in vec4 v_vPosWorldSpace;
layout(location = 2) in vec4 v_vPosEyeSpace;

// OUTPUT
layout(location = 0) out vec4 o_vFragColor;

// UNIFORMS
layout(binding = 2) uniform sampler2D u_tCaustic1Tex;
layout(binding = 3) uniform sampler2D u_tCaustic2Tex;
layout(binding = 4) uniform sampler2D u_tSandTex;

layout(set = 0, binding = 1) uniform LightingBlock {
	vec4 u_vLightPosition;  
	vec4 u_vLightAmbient;   
	vec4 u_vLightDiffuse;   
	float u_fCausticOffset;
	float u_fCausticTiling;
};

float getCausticIntensity(vec4 worldPos)
{
	vec2 sharedUV = (worldPos.xz * u_fCausticTiling);
	float caustic1 = texture(u_tCaustic1Tex, sharedUV + vec2(u_fCausticOffset, 0.0f)).r;
	float caustic2 = texture(u_tCaustic2Tex, sharedUV * 1.3f + vec2(0.0f, u_fCausticOffset)).r;
	return (caustic1 * caustic2);
}

float g_maxFogDist = 75.0f;
vec3 g_fogColor = vec3(0.0f, 0.0f, 0.64f);

float fogIntensity(float dist)
{
	float normalizedDist = clamp((dist / g_maxFogDist), 0.0f, 1.0f);
	// Use a linear falloff to give the ground plane more of a distant look
	return normalizedDist;// * normalizedDist * normalizedDist;
}

void main()
{
    o_vFragColor = texture(u_tSandTex, v_vTexcoord) + getCausticIntensity(v_vPosWorldSpace);
    float fog = fogIntensity(length(v_vPosEyeSpace.xyz));
    o_vFragColor.rgb = mix(o_vFragColor.rgb, g_fogColor, fog);
}
