// TAGRELEASE: PUBLIC
#GLSL_VS
#version 440 core

precision mediump float;

// INPUT
layout(location = 0) in vec2 a_vPosition;     // In BaseShader

// OUTPUT
layout(location = 0) out vec3 v_vCubemapCoord;

layout(set = 0, binding=0) uniform ProjBlock {
	mat4 u_mProjectionMatrix;
	mat4 u_mInverseProjMatrix;
	mat4 u_mViewMatrix;
	mat4 u_mInverseViewMatrix;
};

void main()
{
    gl_Position = vec4(a_vPosition.xy, 0.9999, 1.0);
    
    // Force all depths to just inside the far clip plane. If we put z=0.9999
    // in the vertex attributes, it messes up the cube projection below.
    // Nobbling it here is easier.
    gl_Position.z = 0.9999;

    vec4 vPos = u_mInverseProjMatrix * vec4(a_vPosition.xy, -1.0, 1.0);
    vPos /= vPos.w;
    
    v_vCubemapCoord = (mat3(u_mInverseViewMatrix) * vPos.xyz);
}

#GLSL_FS
#version 440 core

precision mediump float;

// INPUT
layout(location = 0) in vec3 v_vCubemapCoord;

// OUTPUT
layout(location = 0) out vec4 o_vFragColor;

// UNIFORMS
layout(set = 0, binding = 4) uniform sampler2D u_tSandTex;
layout(set = 0, binding = 5) uniform sampler2D u_tGradientTex;

void main()
{
    // o_vFragColor = vec4(0.0f, ((normalize(v_vCubemapCoord.xyz) + 1.0f) * 0.5f).y, 0.0f, 1.0f);
    o_vFragColor = vec4(texture(u_tGradientTex, vec2(0.5f, (normalize(v_vCubemapCoord.xyz).y + 1.0f) * 0.5f)).rgb, 1.0f);

}
